!
! CDDL HEADER START
!
! The contents of this file are subject to the terms of the Common Development
! and Distribution License Version 1.0 (the "License").
!
! You can obtain a copy of the license at
! http://www.opensource.org/licenses/CDDL-1.0.  See the License for the
! specific language governing permissions and limitations under the License.
!
! When distributing Covered Code, include this CDDL HEADER in each file and
! include the License file in a prominent location with the name LICENSE.CDDL.
! If applicable, add the following below this CDDL HEADER, with the fields
! enclosed by brackets "[]" replaced with your own identifying information:
!
! Portions Copyright (c) [yyyy] [name of copyright owner]. All rights reserved.
!
! CDDL HEADER END
!
! Copyright (c) 2015, Regents of the University of Minnesota.
! All rights reserved.
!
! Contributors:
!    Nikhil C. Admal
!    Ellad B. Tadmor
!
! Release: This file is part of the MDStressLab git repository.
!
program main
use mod_particles, only : allocate_atom
use mod_io, only : read_input
use mod_forces, only : set_forces
use mod_rigidity, only : set_rigidity
use mod_output, only : string, output,outputmode
use mod_file, only : GetFreeUnit
use mod_output, only : logunit,logentry,logoutput
use mod_global, only : cmdunit
use mod_qclib, only : downcase
use mod_grid, only : read_grid
use mod_stress, only : set_stress, compute_stress,write_stress
implicit none
!-------------------------------------------------------------------------------
! local variables
!
character(len=80) :: filename,input
character(len=4) :: command
integer :: ierror,i,l
logical :: terminate

!-------------------------------------------------------------------------------
! open the log file
!
filename='sc.log'
logunit=GetFreeUnit()
open(logunit,file=filename,status='replace',form='formatted')

! set current output mode for subroutine output
outputmode = 'normal'

!-------------------------------------------------------------------------------
! setup banner
!
string = ' '; call output
string = '----------------------------------------' //  &
         '------------------------------------'
call output
string = ' '; call output
write(string,'(''M D S t r e s s L a b'')')
call output
string = ' '; call output
string = 'Distribution version 1.0.0 (May 2015)'
call output
string = ' '; call output
string = 'This code is distributed under the ' //       &
         'CDDL open source license'
call output
string = ' '; call output
string = 'Authors: N. C. Admal and E. B. Tadmor ' //    &
         '(University of Minnesota)'
call output
string = ' '; call output
string = 'For more information contact tadmor@umn.edu'
call output
string = ' '; call output
string = '----------------------------------------' //  &
         '------------------------------------'
call output
string = ' '; call output

!-------------------------------------------------------------------------------
! parse input file: strip leading spaces from lines, strip comment lines.
!
filename='sc.cmd'
cmdunit=GetFreeUnit()
open(cmdunit,file=filename,status='replace',form='formatted')
do
    read(5,'(a80)',iostat=ierror) input
    if (ierror<0) exit
    i=1
    do while ((i.le.80))
        if (input(i:i).eq.' ') then 
            i=i+1
        else
            exit
        endif
    enddo
    if (i.le.80) then
        if (input(i:i).eq.'%') cycle ! lines starting with '%' are comments
    endif
    write(cmdunit,'(a)') input(i:80)
enddo
rewind(cmdunit)

! Announce the start of simulation
call logentry
write(string,'(a)') 'Simulation started.'
call logoutput
!-------------------------------------------------------------------------------
! read a card and compare first 4 columns with macro list
!
terminate = .false.
do
    read(cmdunit,'(a80)') input
    command = input(1:4)

    select case(downcase(command))
    !-------------------------------------------------------------------------------
    ! macro 'read' : Read all the input files
    !
    case('read')
        call read_input
    !-------------------------------------------------------------------------------
    ! macro 'grid' : Set up the grid
    !
    case('grid')
        call read_grid
    !-------------------------------------------------------------------------------
    ! macro 'forces' : model settings
    !
    case('pote')
        call set_forces
    !-------------------------------------------------------------------------------
    ! macro 'rigidity' : Interatomic forc settings
    !
    case('uniq')
        call set_rigidity
    !-------------------------------------------------------------------------------
    ! macro 'stress' : Calculate stresses
    !
    case('stre')
        call set_stress
        call compute_stress
        call write_stress
    !-------------------------------------------------------------------------------
    ! macro 'stop' :  Terminate the program
    !
    case('stop')
        terminate = .true.
    end select

    if (terminate) exit
enddo

! Announce program termination
call logentry
write(string,'(a)') 'Simulation terminated.'
call logoutput

! Close all open files and delete the temporary file 'sc.cmd'
do l=7,99 
 close(l)
enddo
call terminate_all
end program
    subroutine terminate_all
    use mod_output, only : string, output,heading
    use mod_particles, only : deallocate_atoms
    use mod_box, only : deallocate_box
    use mod_forces, only : deallocate_forces
    use mod_neigh, only : deallocate_neigh
    use mod_grid, only : deallocate_grid
    implicit none

    heading=.true.
    write(string,'(a)')'**Deallocating data**'; call output
    call deallocate_atoms
    call deallocate_box
    call deallocate_grid
    call deallocate_forces
    end subroutine terminate_all
