!
! CDDL HEADER START
!
! The contents of this file are subject to the terms of the Common Development
! and Distribution License Version 1.0 (the "License").
!
! You can obtain a copy of the license at
! http://www.opensource.org/licenses/CDDL-1.0.  See the License for the
! specific language governing permissions and limitations under the License.
!
! When distributing Covered Code, include this CDDL HEADER in each file and
! include the License file in a prominent location with the name LICENSE.CDDL.
! If applicable, add the following below this CDDL HEADER, with the fields
! enclosed by brackets "[]" replaced with your own identifying information:
!
! Portions Copyright (c) [yyyy] [name of copyright owner]. All rights reserved.
!
! CDDL HEADER END
!
! Copyright (c) 2015, Regents of the University of Minnesota.
! All rights reserved.
!
! Contributors:
!    Nikhil C. Admal
!    Ellad B. Tadmor
!
! Release: This file is part of the MDStressLab git repository.
!
!-------------------------------------------------------------------------------
! MODULE :
!
! type definitions:
! ----------------
! N/A
!
! Real Variable Definitions:
! -------------------------------------
!
! Integer Variable Definitions:
! ----------------------------
!
! logical Variable Definitions:
! ----------------------------
!
! Contains Routines:
! ------------------
!
!
!
!
! SUBROUTINE :
!
! Passed Parameters :-
! -----------------
!
! Non-Obvious Parameter :-
! ---------------------
!
module mod_hardy
use mod_global, only : dp
implicit none
save
private
public hardy,           &
       hardy_p,         &
       hardy_kin,       &
       virial,          &
       virial_p,        &
       virial_kin,      &
       collect_hardy,   &
       write_hardy,     &
       avgsize_hardy
real(kind=dp), dimension(:,:,:), allocatable :: hardy,hardy_p,hardy_kin
real(kind=dp), dimension(:,:,:), allocatable :: virial,virial_p,virial_kin
real(kind=dp) :: avgsize_hardy

contains

    subroutine collect_hardy(l,crossings,cutoff,fij,list,marker1,marker2,tensor,tensork)
    use mod_box, only : dimn,nat,samplesize,pbc
    use mod_grid, only : gpt
    use mod_particles, only : at
    use mod_qclib, only : tensor_prod
    implicit none
    ! Passed variables
    integer,      intent(in) :: l
    logical,      intent(in) :: crossings
    integer,      intent(in) :: marker1(:),marker2(:),list(:)
    real(kind=dp),intent(in) :: fij(:,:),cutoff
    real(kind=dp),intent(out):: tensor(:,:),tensork(:,:)

    real(kind=dp), dimension(dimn) :: r,rij,p1,p2,rp1,rp2,ra,rb
    real(kind=dp), dimension(dimn,2) :: ri,rj
    real(kind=dp), dimension(dimn,dimn) :: rcrossr,vcrossv
    real(kind=dp) :: eff_bond,ddot
    real(kind=dp) :: dist,dist_ri,dist_rj,dist_rij,dist_r,lambda,rsqsp,s,eff_line,rsumsq
    real(kind=dp) :: eps = 1.e-5
    integer :: i,j,k,m,p

    tensor = 0._dp
    tensork = 0._dp
    rsqsp = avgsize_hardy**2
    rsumsq = (avgsize_hardy + cutoff)**2
    do i=1,nat
        r = at(i)%x-gpt(:,l)
        where (abs(r)/samplesize > 0.5_dp .and. pbc)
            r = r-sign(samplesize,r)
        end where
        dist = ddot(dimn,r,1,r,1)
        
        ! Collect the kinetric contribution first
        if (dist .le. avgsize_hardy**2) then
            call tensor_prod(dimn,at(i)%v,at(i)%v,vcrossv)
            if (crossings .eqv. .true.) then
                tensork = tensork + vcrossv * func(sqrt(dist))
            else
                tensork = tensork + vcrossv
            endif

        endif
        ! End of kinetic contribution

        if (dist .le. rsumsq) then
            do p=marker1(i),marker2(i)
            !-------------------------------------------------------------------------------
            !
            !                 ==============================
            !                ||                            ||
            !                ||  _______                   ||
            !                || / .p1   \                  ||  .rp1
            !                ||/ /       \                 || /
            !                ||\/  Avg.  /                 ||/
            !                ||/\ domain/                  |/
            !                |/  \     /                   /|
            !                /|   \___/                   /||
            !               /||                          / ||
            !           rp2. ||                       p2.  ||
            !                ||                            ||
            !                 ==============================
            !
            !
                j = list(p)
                p1 = at(i)%x
                p2 = at(j)%x
                rij = p1-p2
                where (abs(rij)/samplesize > 0.5_dp .and. pbc)
                    rij = rij-sign(samplesize,rij)
                end where
                rp1 = p2 + rij
                rp2 = p1 - rij
                if (maxval(abs(rp1-p1)) .lt. eps) then
                    k=1
                else
                    k=2
                endif

                ri(:,1) = p1  - gpt(:,l)
                rj(:,1) = rp2 - gpt(:,l)
                ri(:,2) = rp1 - gpt(:,l)
                rj(:,2) = p2  - gpt(:,l)

                eff_bond = 0._dp

                ! Initialize dist_rij
                dist_rij = 0._dp

                do m = 1,k
                    dist_ri = ddot(dimn,ri(:,m),1,ri(:,m),1)
                    dist_rj = ddot(dimn,rj(:,m),1,rj(:,m),1)
                    if (dist_ri .gt. rsumsq .or. dist_rj .gt. rsumsq) then
                        cycle
                    endif
                    dist_rij = ddot(dimn,rij,1,rij,1)
                    lambda = ddot(dimn,ri(:,m),1,rij,1)/dist_rij
                    r = ri(:,m) - lambda * rij
                    dist_r = ddot(dimn,r,1,r,1)

                    if (abs(rsqsp - dist_r) .lt. eps) then
                        cycle
                    endif

                    !-------------------------------------------------------------------------------
                    ! If both particles lie within the Hardy sphere
                    !
                    if ( dist_ri .lt. rsqsp .and. dist_rj .lt. rsqsp ) then
                        ra = ri(:,m)
                        rb = rj(:,m)
                        if (crossings .eqv. .false.) then
                            eff_bond = sqrt(dist_rij)
                            cycle
                        endif
                        call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
                        eff_bond = s
                        cycle
                    endif
                    if(crossings .eqv. .true.) then
                        !-------------------------------------------------------------------------------
                        ! If both particles lie outside the Hardy sphere
                        !
                        if( dist_ri .gt. rsqsp .and. dist_rj .gt. rsqsp .and. &
                            lambda .gt. 0._dp .and. lambda .lt. 1._dp   .and. &
                            dist_r .lt. rsqsp ) then

                            eff_line = 2._dp * sqrt(rsqsp - dist_r)
                            ra = r + 0.5_dp * eff_line * (rj(:,m) - ri(:,m))/sqrt(dist_rij)
                            rb = r - 0.5_dp * eff_line * (rj(:,m) - ri(:,m))/sqrt(dist_rij)
                            call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
                            eff_bond = eff_bond + s
                            cycle
                        endif
                        !-------------------------------------------------------------------------------
                        ! If one particle lies inside and other lies outside the hardy sphere
                        !
                        if ( dist_ri .lt. rsqsp .and. dist_rj .gt. rsqsp ) then
                            ra = ri(:,m)
                            if (dist_ri - dist_r .gt. eps) then
                               eff_line = sqrt(rsqsp - dist_r) +                   &
                                               sign(sqrt(dist_ri - dist_r),lambda)
                               rb = ri(:,m) + eff_line*(rj(:,m) - ri(:,m))/sqrt(dist_rij)
                               call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
                               eff_bond = eff_bond + s
                               cycle
                            else
                               eff_line = sqrt(rsqsp - dist_r)
                               rb = ri(:,m) + eff_line*(rj(:,m) - ri(:,m))/sqrt(dist_rij)
                               call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
                               eff_bond = eff_bond + s
                               cycle
                            endif
                        endif

                        if ( dist_ri .gt. rsqsp .and. dist_rj .lt. rsqsp ) then
                            ra = rj(:,m)
                            if (dist_rj - dist_r .gt. eps) then
                               eff_line = sqrt(rsqsp - dist_r) +                     &
                                               sign(sqrt(dist_rj - dist_r),1._dp-lambda)
                               rb = rj(:,m) + eff_line*(ri(:,m) - rj(:,m))/sqrt(dist_rij)
                               call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
                               eff_bond = eff_bond + s
                               cycle
                            else
                               eff_line = sqrt(rsqsp - dist_r)
                               rb = rj(:,m) + eff_line*(ri(:,m) - rj(:,m))/sqrt(dist_rij)
                               call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
                               eff_bond = eff_bond + s
                               cycle
                            endif
                        endif
                    endif
                enddo

                if (eff_bond .gt. 0._dp) then
                    call tensor_prod(dimn,fij(:,p),rij,rcrossr)
                    tensor = tensor-eff_bond*rcrossr/sqrt(dist_rij)
                endif

            enddo
        endif
    enddo
    end subroutine collect_hardy

    !----------------------------------------------------------------------------------------------------
    !subroutine collect_hardy_2d_circ(l,crossings,cutoff,fij,list,marker1,marker2,tensor,tensork)
    !implicit none
    !! Passed variables
    !integer,      intent(in) :: l
    !logical,      intent(in) :: crossings
    !integer,      intent(in) :: marker1(:),marker2(:),list(:)
    !real(kind=dp),intent(in) :: fij(:,:),cutoff
    !real(kind=dp),intent(out):: tensor(:,:),tensork(:,:)
    !end subroutine collect_hardy_2d_circ
    !
    !subroutine collect_hardy_3d_sphere(l,crossings,cutoff,fij,list,marker1,marker2,tensor,tensork)
    !use mod_box, only : dimn,nat,samplesize,pbc
    !use mod_grid, only : gpt
    !use mod_particles, only : at
    !use mod_qclib, only : tensor_prod
    !implicit none
    !! Passed variables
    !integer,      intent(in) :: l
    !logical,      intent(in) :: crossings
    !integer,      intent(in) :: marker1(:),marker2(:),list(:)
    !real(kind=dp),intent(in) :: fij(:,:),cutoff
    !real(kind=dp),intent(out):: tensor(:,:),tensork(:,:)
    !
    !real(kind=dp), dimension(dimn) :: r,rij,p1,p2,rp1,rp2,ra,rb
    !real(kind=dp), dimension(dimn,2) :: ri,rj
    !real(kind=dp), dimension(dimn,dimn) :: rcrossr,vcrossv
    !real(kind=dp) :: eff_bond,ddot
    !real(kind=dp) :: dist,dist_ri,dist_rj,dist_rij,dist_r,lambda,rsqsp,s,eff_line,rsumsq
    !real(kind=dp) :: eps = 1.e-5
    !integer :: i,j,k,m,p
    !
    !tensor = 0._dp
    !tensork = 0._dp
    !rsqsp = avgsize_hardy**2
    !rsumsq = (avgsize_hardy + cutoff)**2
    !do i=1,nat
    !    r = at(i)%x-gpt(:,l)
    !    where (abs(r)/samplesize > 0.5_dp .and. pbc)
    !        r = r-sign(samplesize,r)
    !    end where
    !    dist = ddot(dimn,r,1,r,1)
    !
    !    ! Collect the kinetric contribution first
    !    if (dist .le. avgsize_hardy**2) then
    !        call tensor_prod(dimn,at(i)%v,at(i)%v,vcrossv)
    !        if (crossings .eqv. .true.) then
    !            tensork = tensork + vcrossv * func(sqrt(dist))
    !        else
    !            tensork = tensork + vcrossv
    !        endif
    !
    !    endif
    !    ! End of kinetic contribution
    !
    !    if (dist .le. rsumsq) then
    !        do p=marker1(i),marker2(i)
    !        !-------------------------------------------------------------------------------
    !        !
    !        !                 ==============================
    !        !                ||                            ||
    !        !                ||  _______                   ||
    !        !                || / .p1   \                  ||  .rp1
    !        !                ||/ /       \                 || /
    !        !                ||\/  Avg.  /                 ||/
    !        !                ||/\ domain/                  |/
    !        !                |/  \     /                   /|
    !        !                /|   \___/                   /||
    !        !               /||                          / ||
    !        !           rp2. ||                       p2.  ||
    !        !                ||                            ||
    !        !                 ==============================
    !        !
    !        !
    !            j = list(p)
    !            p1 = at(i)%x
    !            p2 = at(j)%x
    !            rij = p1-p2
    !            where (abs(rij)/samplesize > 0.5_dp .and. pbc)
    !                rij = rij-sign(samplesize,rij)
    !            end where
    !            rp1 = p2 + rij
    !            rp2 = p1 - rij
    !            if (maxval(abs(rp1-p1)) .lt. eps) then
    !                k=1
    !            else
    !                k=2
    !            endif
    !
    !            ri(:,1) = p1  - gpt(:,l)
    !            rj(:,1) = rp2 - gpt(:,l)
    !            ri(:,2) = rp1 - gpt(:,l)
    !            rj(:,2) = p2  - gpt(:,l)
    !
    !            eff_bond = 0._dp
    !
    !            do m = 1,k
    !                dist_ri = ddot(dimn,ri(:,m),1,ri(:,m),1)
    !                dist_rj = ddot(dimn,rj(:,m),1,rj(:,m),1)
    !                if (dist_ri .gt. rsumsq .or. dist_rj .gt. rsumsq) then
    !                    cycle
    !                endif
    !                dist_rij = ddot(dimn,rij,1,rij,1)
    !                lambda = ddot(dimn,ri(:,m),1,rij,1)/dist_rij
    !                r = ri(:,m) - lambda * rij
    !                dist_r = ddot(dimn,r,1,r,1)
    !
    !                if (abs(rsqsp - dist_r) .lt. eps) then
    !                    cycle
    !                endif
    !
    !                !-------------------------------------------------------------------------------
    !                ! If both particles lie within the Hardy sphere
    !                !
    !                if ( dist_ri .lt. rsqsp .and. dist_rj .lt. rsqsp ) then
    !                    ra = ri(:,m)
    !                    rb = rj(:,m)
    !                    if (crossings .eqv. .false.) then
    !                        eff_bond = sqrt(dist_rij)
    !                        cycle
    !                    endif
    !                    call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
    !                    eff_bond = s
    !                    cycle
    !                endif
    !                if(crossings .eqv. .true.) then
    !                    !-------------------------------------------------------------------------------
    !                    ! If both particles lie outside the Hardy sphere
    !                    !
    !                    if( dist_ri .gt. rsqsp .and. dist_rj .gt. rsqsp .and. &
    !                        lambda .gt. 0._dp .and. lambda .lt. 1._dp   .and. &
    !                        dist_r .lt. rsqsp ) then
    !
    !                        eff_line = 2._dp * sqrt(rsqsp - dist_r)
    !                        ra = r + 0.5_dp * eff_line * (rj(:,m) - ri(:,m))/sqrt(dist_rij)
    !                        rb = r - 0.5_dp * eff_line * (rj(:,m) - ri(:,m))/sqrt(dist_rij)
    !                        call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
    !                        eff_bond = eff_bond + s
    !                        cycle
    !                    endif
    !                    !-------------------------------------------------------------------------------
    !                    ! If one particle lies inside and other lies outside the hardy sphere
    !                    !
    !                    if ( dist_ri .lt. rsqsp .and. dist_rj .gt. rsqsp ) then
    !                        ra = ri(:,m)
    !                        if (dist_ri - dist_r .gt. eps) then
    !                           eff_line = sqrt(rsqsp - dist_r) +                   &
    !                                           sign(sqrt(dist_ri - dist_r),lambda)
    !                           rb = ri(:,m) + eff_line*(rj(:,m) - ri(:,m))/sqrt(dist_rij)
    !                           call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
    !                           eff_bond = eff_bond + s
    !                           cycle
    !                        else
    !                           eff_line = sqrt(rsqsp - dist_r)
    !                           rb = ri(:,m) + eff_line*(rj(:,m) - ri(:,m))/sqrt(dist_rij)
    !                           call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
    !                           eff_bond = eff_bond + s
    !                           cycle
    !                        endif
    !                    endif
    !
    !                    if ( dist_ri .gt. rsqsp .and. dist_rj .lt. rsqsp ) then
    !                        ra = rj(:,m)
    !                        if (dist_rj - dist_r .gt. eps) then
    !                           eff_line = sqrt(rsqsp - dist_r) +                     &
    !                                           sign(sqrt(dist_rj - dist_r),1._dp-lambda)
    !                           rb = rj(:,m) + eff_line*(ri(:,m) - rj(:,m))/sqrt(dist_rij)
    !                           call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
    !                           eff_bond = eff_bond + s
    !                           cycle
    !                        else
    !                           eff_line = sqrt(rsqsp - dist_r)
    !                           rb = rj(:,m) + eff_line*(ri(:,m) - rj(:,m))/sqrt(dist_rij)
    !                           call qtrap(ra,rb,.false.,0._dp,(/0._dp,0._dp,0._dp/),s)
    !                           eff_bond = eff_bond + s
    !                           cycle
    !                        endif
    !                    endif
    !                endif
    !            enddo
    !
    !            if (eff_bond .gt. 0._dp) then
    !                call tensor_prod(dimn,fij(:,p),rij,rcrossr)
    !                tensor = tensor-eff_bond*rcrossr/sqrt(dist_rij)
    !            endif
    !
    !        enddo
    !    endif
    !enddo
    !end subroutine collect_hardy_3d_sphere
    !
    !
    subroutine write_hardy(flag)
    use mod_global, only : PI
    use mod_file, only : GetFreeUnit
    use mod_grid, only : ngrid,gpt
    use mod_output, only : string, output
    use mod_rigidity, only : project

    implicit none
    integer, intent(in) :: flag

    character(len=80) :: filename
    integer :: unit,i
    real(kind=dp) :: avgvol

    avgvol = 4._dp*PI*(avgsize_hardy**3)/3._dp

    if (flag .eq. 1) then
        ! Write potential and kinetic parts of hardy stress
        filename = 'hardy.str'
        unit = GetFreeUnit()
        open(unit,file=filename,status='replace',form='formatted')
        write(string,'(a,a)') 'Wring Hardy stress tensor to  file', &
                               trim(filename); call output
        hardy   = hardy/avgvol
        do i=1,ngrid
            write(unit,8010)  gpt(1,i),         &
                              gpt(2,i),         &
                              gpt(3,i),         &
                              hardy(:,:,i)
        enddo
        filename = 'hardy_kin.str'
        unit = GetFreeUnit()
        open(unit,file=filename,status='replace',form='formatted')
        write(string,'(a,a)') 'Wring Kinetic part of Hardy stress tensor to  file', &
                               trim(filename); call output
        hardy_kin   = hardy_kin/avgvol
        do i=1,ngrid
            write(unit,8010)  gpt(1,i),         &
                              gpt(2,i),         &
                              gpt(3,i),         &
                              hardy_kin(:,:,i)
        enddo

        ! Write the potential part of projected Hardy stress
        if (project) then
            filename = 'hardy_p.str'
            unit = GetFreeUnit()
            open(unit,file=filename,status='replace',form='formatted')
            write(string,'(a,a)') 'Wring projected Hardy stress tensor to  file', &
                                   trim(filename); call output
            hardy_p = hardy_p/avgvol

            do i=1,ngrid
                write(unit,8010) gpt(1,i),         &
                                  gpt(2,i),         &
                                  gpt(3,i),         &
                                  hardy_p(:,:,i)
            enddo
        endif

    ! Repeat the above for virial stress
    else
        filename = 'virial.str'
        unit = GetFreeUnit()
        open(unit,file=filename,status='replace',form='formatted')
        write(string,'(a,a)') 'Wring Virial stress tensor to  file', &
                               trim(filename); call output
        virial= virial/avgvol
        do i=1,ngrid
            write(unit,8010)  gpt(1,i),         &
                              gpt(2,i),         &
                              gpt(3,i),         &
                              virial(:,:,i)
        enddo
        filename = 'virial_kin.str'
        unit = GetFreeUnit()
        open(unit,file=filename,status='replace',form='formatted')
        write(string,'(a,a)') 'Wring Kinetic part of virial stress tensor to  file', &
                               trim(filename); call output
        virial_kin = virial_kin/avgvol
        do i=1,ngrid
            write(unit,8010)  gpt(1,i),         &
                              gpt(2,i),         &
                              gpt(3,i),         &
                              virial_kin(:,:,i)
        enddo

        if (project) then
            filename = 'virial_p.str'
            unit = GetFreeUnit()
            open(unit,file=filename,status='replace',form='formatted')
            write(string,'(a,a)') 'Wring the projected Virial stress tensor to  file', &
                                   trim(filename); call output
            virial_p = virial_p/avgvol

            do i=1,ngrid
                write(unit,8010)  gpt(1,i),         &
                                  gpt(2,i),         &
                                  gpt(3,i),         &
                                  virial_p(:,:,i)
            enddo
        endif
    endif

    8010 format(1x,12f25.15)

    end subroutine write_hardy

    subroutine qtrap(ra,rb,calc_mom,dist_rij,rj,s)
    use mod_box, only : dimn
    implicit none
    logical,intent(in) :: calc_mom
    real(kind=dp),intent(in) :: ra(dimn),rb(dimn),rj(dimn)
    real(kind=dp),intent(in) :: dist_rij
    real(kind=dp),intent(out) :: s
    real(kind=dp), parameter :: eps=1.e-3
    integer, parameter:: jmax=20
    real(kind=dp) :: olds
    integer :: j
    olds = -1.e30
    do j=1,jmax
        call trapzd(ra,rb,calc_mom,s,j,dist_rij,rj)
        if (abs(s-olds) .lt. eps*abs(olds)) then
            !write(*,*) "Number of iterations: 2 power :",j
            return
        endif
        olds=s
    enddo
    end subroutine qtrap

    subroutine trapzd(ra,rb,calc_mom,s,na,dist_rij,rj)
    use mod_box, only : dimn
    implicit none
    real(kind=dp),intent(in) :: ra(dimn),rb(dimn),rj(dimn)
    real(kind=dp),intent(in):: dist_rij
    real(kind=dp) :: x(dimn)
    real(kind=dp) :: da,db,dx,d,del,sum,lam,ddot
    integer :: na,tnm,j
    integer, save :: it
    real(kind=dp) :: s
    logical, intent(in) :: calc_mom

    da = sqrt(ddot(dimn,ra,1,ra,1))
    db = sqrt(ddot(dimn,rb,1,rb,1))
    d = sqrt(ddot(dimn,ra-rb,1,ra-rb,1))
    if (na .eq. 1) then
        if(calc_mom .eqv. .true.) then
            s = 0.5_dp*d*(func(db)*d)
        else
            s = 0.5_dp*d*(func(da) + func(db))
        endif
        it=1
    else
        tnm = it
        del=d/tnm
        x = ra + 0.5_dp*del*(rb - ra)/d
        sum=0._dp
        do j=1,it
            dx = sqrt(ddot(dimn,x,1,x,1))
            lam = sqrt(ddot(dimn,x - rj,1,x-rj,1) / dist_rij)
            if(calc_mom .eqv. .true.) then
                sum=sum+func(dx) * (1._dp+lam)**2
            else
                sum=sum+func(dx)
            endif
            x = x + del*(rb - ra)/d
        enddo
        s = 0.5_dp*(s+d*sum/tnm)
        it=2*it
    endif
    return
    end subroutine trapzd
            
    function func(r)
    use mod_global, only : PI
    implicit none
    real(kind=dp),intent(in) :: r
    real(kind=dp) :: func

    real(kind=dp) :: factor,eps
    eps = 0.12_dp * avgsize_hardy
    factor = 1._dp + (3._dp/PI**2 - 0.5_dp)*(eps/avgsize_hardy)**3 +    &
            (1.5_dp-6._dp/PI**2)*(eps/avgsize_hardy)**2 - 1.5_dp*(eps/avgsize_hardy)
    factor = 1._dp/factor
    if (r .lt. avgsize_hardy-eps) then
        func = factor
    elseif (r .gt. avgsize_hardy-eps .and. r .lt. avgsize_hardy) then
        func = 0.5_dp*factor*(1._dp - cos((r-avgsize_hardy)*PI/eps))
    else
        func = 0._dp
    endif
    end function func

end module mod_hardy
