!
! CDDL HEADER START
!
! The contents of this file are subject to the terms of the Common Development
! and Distribution License Version 1.0 (the "License").
!
! You can obtain a copy of the license at
! http://www.opensource.org/licenses/CDDL-1.0.  See the License for the
! specific language governing permissions and limitations under the License.
!
! When distributing Covered Code, include this CDDL HEADER in each file and
! include the License file in a prominent location with the name LICENSE.CDDL.
! If applicable, add the following below this CDDL HEADER, with the fields
! enclosed by brackets "[]" replaced with your own identifying information:
!
! Portions Copyright (c) [yyyy] [name of copyright owner]. All rights reserved.
!
! CDDL HEADER END
!
! Copyright (c) 2015, Regents of the University of Minnesota.
! All rights reserved.
!
! Contributors:
!    Nikhil C. Admal
!    Ellad B. Tadmor
!
! Release: This file is part of the MDStressLab git repository.
!
!-------------------------------------------------------------------------------
! MODULE mod_cayley :
!
! type definitions:
! ----------------
! N/A
!
! Real Variable Definitions:
! -------------------------------------
! 
!
! Integer Variable Definitions:
! ----------------------------
!
! logical Variable Definitions: N/A
! ----------------------------
!
! Contains Routines:
! ------------------
!
# include "KIM_API_status.h"
#define THIS_FILE_NAME __FILE__
#define TRUEFALSE(TRUTH) merge(1,0,(TRUTH))
module mod_kim

use, intrinsic :: iso_c_binding
use KIM_API_F03
use mod_types
use mod_global, only : dp
implicit none
save
public
type(c_ptr) :: pkim
integer(c_int) :: ier,idum

contains
    subroutine setup_KIM_API_object(testkimfile, modelname,  &
                                    dimn,nat,nsp,at,energy,fi,rcutoff,samplesize)
    !-------------------------------------------------------------------------------
    !
    ! free_KIM_API_object : Deallocate storage and destroy KIM API object
    !
    !-------------------------------------------------------------------------------
    implicit none

    !-- Transferred variables
    character(len=KIM_KEY_STRING_LENGTH),intent(in) :: testkimfile
    character(len=KIM_KEY_STRING_LENGTH),intent(in) :: modelname
    integer,                             intent(in) :: dimn
    integer,target,                      intent(in) :: nat
    integer,target,                      intent(in) :: nsp
    type(atom),                          intent(in) :: at(nat)
    real(kind=dp),target,                intent(in) :: energy,fi(dimn,nat)

    integer(c_int), parameter   :: SizeOne = 1
    real(kind=dp),  intent(out) :: rcutoff
    real(kind=dp),  intent(in)  :: samplesize(dimn)

    !-- Local variables
    real(c_double),pointer  :: cutoff;             type(c_ptr) :: pcutoff
    real(c_double),pointer  :: coor(:,:);          type(c_ptr) :: pcoor
    real(c_double),pointer  :: boxsize(:);         type(c_ptr) :: pboxsize
    integer(c_int),pointer  :: particleTypes(:);   type(c_ptr) :: pparticleTypes

    integer              :: i

    call Write_KIM_descriptor_file(testkimfile)

    ! Initialize KIM API object
    !
    ier = kim_api_file_init(pkim, testkimfile, modelname)
    if (ier.lt.KIM_STATUS_OK) then
     idum = kim_api_report_error(__LINE__, THIS_FILE_NAME, "kim_api_file_init", ier)
     stop
    endif

    ! Allocate memory via the KIM system
    call kim_api_allocate(pkim, nat, nsp, ier)
    if (ier.lt.KIM_STATUS_OK) then
        idum = kim_api_report_error(__LINE__, THIS_FILE_NAME, "kim_api_allocate", ier)
        stop
    endif

    ! call model's init routine
    !
    ier = kim_api_model_init(pkim)
    if (ier.lt.KIM_STATUS_OK) then
     idum = kim_api_report_error(__LINE__, __FILE__, "kim_api_model_init", ier)
     stop
    endif

    call kim_api_getm_data( pkim, ier,            &
        "coordinates",      pcoor,              1,  &
        "particleSpecies",  pparticleTypes,     1,  &
        "boxSideLengths",   pboxsize,           1)
    if (ier.lt.KIM_STATUS_OK) then
     idum = kim_api_report_error(__LINE__, __FILE__, "kim_api_getm_data", ier)
     stop
    endif

    call c_f_pointer(pcoor,coor,[dimn,nat])
    call c_f_pointer(pboxsize,boxsize,[dimn])
    call c_f_pointer(pparticleTypes,particleTypes,[nat])


    boxsize = samplesize
    coor = 0._dp
    do i=1,nat
        coor(:,i) = at(i)%x(:)
        particleTypes(i) = kim_api_get_species_code(pkim,at(i)%sp_ptr%id,ier)
    enddo

    call kim_api_setm_data( pkim,ier,                              &
        "numberOfParticles",  SizeOne,         c_loc(nat),      1, &
        "numberContributingParticles", SizeOne,c_loc(nat),      1, &
        "forces",             SizeOne,         c_loc(fi),       1, &
        "energy",             SizeOne,         c_loc(energy),   1, &
        "numberOfSpecies",SizeOne,             c_loc(nsp),      1)
    if (ier.lt.KIM_STATUS_OK) then
     idum = kim_api_report_error(__LINE__, __FILE__, "kim_api_setm_data", ier)
     stop
    endif

    pcutoff = kim_api_get_data(pkim,"cutoff",ier)
    if (ier.lt.KIM_STATUS_OK) then
     idum = kim_api_report_error(__LINE__, __FILE__, "kim_api_get_data", ier)
     stop
    endif
    call c_f_pointer(pcutoff,cutoff)

    rcutoff = cutoff
    return
    end subroutine setup_KIM_API_object

    subroutine setup_kim_access(pcalc_fij,pget_neigh)
    !-------------------------------------------------------------------------------
    !
    ! setup_neighborlist_no_Rij_KIM_access :
    !
    !    Store necessary pointers in KIM API object to access the neighbor list
    !    data and methods.
    !
    !-------------------------------------------------------------------------------
    implicit none

    !-- Transferred variables
    type(c_funptr), intent(in) :: pcalc_fij,pget_neigh

    !-- Local variables
    integer(c_int), parameter :: SizeOne = 1

    ! store pointers to neighbor list object and access function
    !
    call kim_api_setm_data(pkim, ier,                &
       "get_neigh",     SizeOne, pget_neigh,    1,     &
       "process_dEdr",  SizeOne, pcalc_fij,     1)

    if (ier.lt.KIM_STATUS_OK) then
     idum = kim_api_report_error(__LINE__, __FILE__, "kim_api_setm_data", ier)
     stop
    endif
    return
    end subroutine setup_kim_access

    subroutine model_compute
    implicit none

    ier = KIM_API_model_compute(pkim)
    if (ier.lt.KIM_STATUS_OK) then
        idum = kim_api_report_error(__LINE__, __FILE__, "kim_api_model_compute", ier)
        stop
    endif
    end subroutine model_compute

    subroutine free_KIM_API_object
    !-------------------------------------------------------------------------------
    !
    ! free_KIM_API_object : Deallocate storage and destroy KIM API object
    !
    !-------------------------------------------------------------------------------
    implicit none

    ! call the model destroy function
    !
    ier = kim_api_model_destroy(pkim)
    if (ier.lt.KIM_STATUS_OK) then
     idum = kim_api_report_error(__LINE__, __FILE__, "kim_api_model_destroy", ier)
     stop
    endif

    ! free all KIM API object storage
    !
    call kim_api_free(pkim, ier)
    if (ier.lt.KIM_STATUS_OK) then
     idum = kim_api_report_error(__LINE__, __FILE__, "kim_api_free", ier)
     stop
    endif

    return

    end subroutine free_KIM_API_object

!-------------------------------------------------------------------------------
!
!  Write KIM descriptor file for MDStressLab
!
!-------------------------------------------------------------------------------
subroutine Write_KIM_descriptor_file(kim_descriptor_file)
use mod_box,   only : sp_list,nsp
use mod_file, only : GetFreeUnit
implicit none

!-- Transferred variables
character(len=80), intent(in)  :: kim_descriptor_file

!-- Local variables
integer :: i,dunit
character(len=10000):: kim_descriptor_file_string
character(len=103) :: divider
character(len=1)   :: cr
character(len=52)  :: type_line

! Define frequently used variables
!
cr = char(10)
divider = '#######################################################################################################'

! Write Minimol descriptor file into string kim_descriptor
!
kim_descriptor_file_string = &
  divider                                                                            // cr // &
  '#'                                                                                // cr // &
  '# Copyright 2011 Ellad B. Tadmor, Ryan S. Elliott, and James P. Sethna'           // cr // &
  '# All rights reserved.'                                                           // cr // &
  '#'                                                                                // cr // &
  '# Author: Automatically generated by MDStressLab'                                 // cr // &
  '#'                                                                                // cr // &
  '#'                                                                                // cr // &
  '# See KIM_API/standard.kim for documentation about this file'                     // cr // &
  '#'                                                                                // cr // &
  divider                                                                            // cr // &
                                                                                        cr // &
                                                                                        cr // &
  'KIM_API_Version  := 1.7.1'                                                         // cr // &
                                                                                        cr // &
  'Unit_length      := A'                                                            // cr // &
  'Unit_energy      := eV'                                                           // cr // &
  'Unit_charge      := e'                                                            // cr // &
  'Unit_temperature := K'                                                            // cr // &
  'Unit_time        := ps'                                                           // cr // &
                                                                                        cr // &
                                                                                        cr // &
  divider                                                                            // cr // &
  'PARTICLE_SPECIES:'                                                  // cr // &
  '# Symbol/name           Type                    code'                             // cr

do i = 1,nsp
  write(type_line,'(a2,20x,''spec'',20x,i4)') sp_list(i),i
  kim_descriptor_file_string = trim(kim_descriptor_file_string) // type_line // cr
enddo

kim_descriptor_file_string = trim(kim_descriptor_file_string) // &
                                                                                        cr // &
                                                                                        cr // &
  divider                                                                            // cr // &
  'CONVENTIONS:'                                                                     // cr // &
  '# Name                  Type'                                                     // cr // &
                                                                                        cr // &
  'OneBasedLists           flag'                                                     // cr // &
                                                                                        cr // &
  'Neigh_LocaAccess        flag'                                                     // cr // &
                                                                                        cr // &
  'MI_OPBC_H               flag'                                                     // cr // &
                                                                                        cr // &
                                                                                        cr // &
  divider                                                                            // cr // &
  'MODEL_INPUT:'                                                                     // cr // &
  '# Name                        Type         Unit       Shape              requirements' // cr // &
  'numberOfParticles             integer      none       []'                         // cr // &
                                                                                        cr // &
  'numberContributingParticles   integer      none       []'                         // cr // &
                                                                                        cr // &
  'numberOfSpecies               integer      none       []'                         // cr // &
                                                                                        cr // &
  'particleSpecies               integer      none       [numberOfParticles]'        // cr // &
                                                                                        cr // &
  'coordinates                   double       length     [numberOfParticles,3]'      // cr // &
                                                                                        cr // &
  'process_dEdr                  method       none       []'                         // cr // &
                                                                                        cr // &
  'get_neigh                     method       none       []'                         // cr // &
                                                                                        cr // &
  'neighObject                   pointer      none       []'                         // cr // &
                                                                                        cr // &
  'boxSideLengths                double       length     [3]'                        // cr // &
                                                                                        cr // &
                                                                                        cr // &
  divider                                                                            // cr // &
  'MODEL_OUTPUT:'                                                                    // cr // &
  '# Name                        Type         Unit       Shape              requirements' // cr // &
                                                                                        cr // &
  'destroy                       method       none       []'                         // cr // &
                                                                                        cr // &
  'compute                       method       none       []'                         // cr // &
                                                                                        cr // &
  'reinit                        method       none       []'                         // cr // &
                                                                                        cr // &
  'cutoff                        double       length     []'                         // cr // &
                                                                                        cr // &
  'energy                        double       energy     []'                         // cr // &
                                                                                        cr // &
  'particleEnergy                double       energy     [numberOfParticles]'        // cr // &
                                                                                        cr // &
  'forces                        double       force      [numberOfParticles,3]'      // cr // &
                                                                                        cr // &
  divider                                                                            // cr 
  !// char(0)

! Write the string kim_descriptor_file_string to descriptor.kim 
dunit=GetFreeUnit()
open(dunit,file=trim(kim_descriptor_file),action='write')
write(dunit,'(a)') trim(kim_descriptor_file_string)
close(dunit)

return

end subroutine Write_KIM_descriptor_file
end module mod_kim
