!
! CDDL HEADER START
!
! The contents of this file are subject to the terms of the Common Development
! and Distribution License Version 1.0 (the "License").
!
! You can obtain a copy of the license at
! http://www.opensource.org/licenses/CDDL-1.0.  See the License for the
! specific language governing permissions and limitations under the License.
!
! When distributing Covered Code, include this CDDL HEADER in each file and
! include the License file in a prominent location with the name LICENSE.CDDL.
! If applicable, add the following below this CDDL HEADER, with the fields
! enclosed by brackets "[]" replaced with your own identifying information:
!
! Portions Copyright (c) [yyyy] [name of copyright owner]. All rights reserved.
!
! CDDL HEADER END
!
! Copyright (c) 2015, Regents of the University of Minnesota.
! All rights reserved.
!
! Contributors:
!    Nikhil C. Admal
!    Ellad B. Tadmor
!
! Release: This file is part of the MDStressLab git repository.
!
!****************************************************************************
!**
!**  MODULE mod_output: contains routines used for output information to
!**                    output stream and log file.
!**
!**  Variable Definitions: 
!**  ---------------------
!**
!**  Integer Variables:
!**    line                 -  macro line number being executed
!**    level                -  current loop nesting level in subroutine pmacr.
!**                            Also used by subroutine output to set the
!**                            indentation.
!**    logunit              -  file handle for log file
!**    lentry               -  log file entry counter
!**    nerror               -  number of lines in the error message sent
!**                            to subroutine erroroutput in error()
!**                            
!**  Character variables:
!**    prefix               -  string prefix to be printed before command
!**                            output. Contains macro commands or appropriate
!**                            indentation.
!**    string               -  variable containing text string to be printed
!**                            after prefix
!**    error()              -  text of error message sent to subroutine 
!**                            erroroutput
!**    spaces               -  string full of blanks used for padding
!**
!**    outputmode           -  Defines current output mode:
!**                            normal = regular output
!**                            macros = output generated during the macro
!**                                     execution phase in subroutine pmacr
!**                                     (indented macro form is adopted).
!**
!**  Logical variables
!**    doubleoff            -  deactivate once the blank line introduced
!**                            above every macro statement called from pmacr
!**                            during outputmode='macros'
!**                            (used for the macro statement listing)
!**   
!**    heading              -  activate heading format once in 
!**                            outputmode='normal' (used for headings).
!**
!**
!**  Contains Routines:
!**  ------------------
!**
!**    output              : send text to output stream in a controlled format
!**    logentry            : send text to log file in a controlled format
!**
!****************************************************************************

Module mod_output

save
public

!* Variable Definitions
   integer, parameter :: nerrmax=25
   integer line,level,logunit,nerror
   character(len=160) string,error(nerrmax)
   character(len=15) prefix
   character(len=80), parameter :: spaces=''
   character(len=6) outputmode
   integer :: lentry = 0
   logical :: doubleoff = .false.
   logical :: heading = .false.

Contains

!****************************************************************************

      subroutine output
      use mod_nonstandard, only : qcflush
      implicit none

!-- Local variables
      select case(outputmode)

      case('normal')
 
      if (heading) then
         write(6,*)
         write(6,'(1x,a)') trim(string)
         heading = .false.
      else
         write(6,'(3x,a)') trim(string)
      endif
      call qcflush(6)

      end select

      return

      end subroutine output

!****************************************************************************

      subroutine logentry
      use mod_nonstandard, only : qcflush
      implicit none

!-- Local variables
      integer values(8)

      values = 0
      call date_and_time(values=values)
      lentry = lentry + 1
      write(logunit,*)
      write(logunit,100) 'Log Entry ',lentry,values(1:3),values(5:6)
 100  format('--',a,i4,6x,i4,'-',i2.2,'-',i2.2,' ',i2.2,':',i2.2)
      call qcflush(logunit)

      end subroutine logentry

!****************************************************************************

      subroutine logoutput
      use mod_nonstandard, only : qcflush
      implicit none

      write(logunit,'(a)') trim(string)
      call qcflush(logunit)

      end subroutine logoutput

      subroutine erroroutput
      implicit none

!-- Local variables
      integer i

      ! send error message to output stream
      write(6,'(///)')
      write(6,'(2a)') '*** ERROR: ',trim(error(1))
      do i=2,nerror
         write(6,'(11x,a)') trim(error(i))
      enddo
      write(6,*)
      write(6,'(a)')'QC simulation terminated.'
      write(6,*)

      ! send message to log
      call logentry
      string = 'QC simulation terminated due to error.'
      call logoutput
      string = '*** ERROR: ' // trim(error(1))
      call logoutput
      do i=2,nerror
         string = spaces(:11) // trim(error(i))
         call logoutput
      enddo

      end subroutine erroroutput

!****************************************************************************

end module mod_output
